@@ -33,7 +33,7 @@ no_selfinf: non-indexed lines that will not be considered for surface gradient o
 Applicable for lines in wing tips, so as to reduce the superestimation of crosswash caused by the absence of a wing closing patch
 nvector: its normal vector, calculated in Solid.end_preprocess() method
 S: its area, calculated in Solid.end_preprocess() method
-colpoint: its collocation point (center), calculated in Solid.end_preprocess() method
+colpoint: its collocation point (center), calculated in Solid.end_preprocess() method and recalculated in PG correction using Solid.panel_calcSn() method
 
 =======
 methods
@@ -45,6 +45,57 @@ nocirc_enforce
 paneller.Panel.__init__.__doc__='''__init__(lines): initializes a Panel object with \'lines\' as its original line list. All other lists are initialized empty'''
 paneller.Panel.nocirc_enforce.__doc__='''nocirc_enforce(linind): adds line linind (inputed and kept as non-signalled) to no_selfinf list'''
 
+paneller.WakeLine.__doc__='''
+
+Class containing info about a wake line. Used in Solid class for free wake calculations
+
+=========
+variables
+=========
+
+ind: the line\'s signed index in Solid class instance
+nabut: number of panels to which the wake line is associated. Used for averaging the local air velocity vector based on values for adjacent panels
+v: velocity vector indicating local velocity based on which to define
+inverted: whether the line is oriented, within its associated Solid class instance, in the positive direction of x-axis.
+updateme: whether the line\'s geometry has already been updated in the current wake rollup iteration
+
+=======
+methods
+=======
+
+__init__
+addvel
+'''
+
+paneller.WakeLine.__init__.__doc__='''__init__(ind): constructor initializing a wake line with index ind in
+associated Solid class instance'''
+paneller.WakeLine.addvel.__doc__='''addvel(v): add velocity in argument v to wake line\'s calculated velocity
+for rollup (variable WakeLine.v) and add 1 to WakeLine.nabut. I. E. method to be executed to compute mean velocity
+between the one or two adjacent wake panels'''
+
+paneller.WakePanel.__doc__='''
+
+Class encompassing information about a wake panel
+
+=========
+variables
+=========
+
+center: point array locating the panel\'s center, in which velocities for wake rollup are calculated
+wl: pointer to WakeLine class instance located at the panel\'s left side
+wr: " right side
+v: air velocity calculated at the wake panel\'s center
+
+=======
+methods
+=======
+
+__init__
+'''
+
+paneller.WakePanel.__doc__='''__init__(wl, wr, center): instantiate a wake panel with WakeLine instances
+wl (left wake line) and wr (right wake line), with center indicated by argument (point array)'''
+
 paneller.Solid.__doc__='''
 
 Class to contain overall info about a solid, including a list of its component panels. To be instantiated with every case study
@@ -77,6 +128,7 @@ aicm3: same for each panel (shape (3, npanels, npanels), aicm3[i, :, :]=aicm3_li
 aicm: aerodynamic influence coefficient matrix based on vortex rings and the application of Neumann contour conditions
 selfinf_mat_x, y, z: sparse csr matrixes so that -gradGamma=(selfinf_mat_x@solution_line, selfinf_mat_y@solution_line, selfinf_mat_z@solution_line)
 iaicm: inverse of Solid.aicm, using Thikhonov regularization if non-zero damper argument is provided to Solid.solve
+wakestrips: list of lists. Each sublist indicates a set of indexed wake line indexes. To be accessed in order to identify wake lines for rollup calculations
 
 =======
 methods
@@ -161,9 +213,10 @@ paneller.Solid.gen_selfinf_mat.__doc__='''gen_selfinf_mat(): generates variables
 to compute local vorticity surface gradient, used for surface velocity deduction as deduced by Ashok Srivastava in his papers'''
 paneller.Solid.gen_selfinf.__doc__='''gen_selfinf(): computes local vorticity gradient and adds its influence on local disturbance velocity as computed by Ashok Srivastava in his papers on Vortex Panel Method.
 Adds the computed contribution to Solid.delphi'''
-paneller.Solid.solve.__doc__='''solve(damper=0.0, target=np.array([])): computes solution and surface forces, with local panel transpiration (to enable viscid-inviscid coupling) input to \'target\'.
+paneller.Solid.solve.__doc__='''solve(damper=0.0, target=np.array([])wakeiter=0, Uinf=1.0, a=0.0, b=0.0, p=0.0, q=0.0, r=0.0, tolerance=1e-5, echo=True):
+computes solution and surface forces, with local panel transpiration (to enable viscid-inviscid coupling) input to \'target\'.
 if len(target)==0: target=np.zeros(npanels). A Thikhonov regularization is performed with damping coefficient \'damper\' if damper floating point kwarg is set as non-zero, thus solving even an ill-conditioned 
-AIC matrix'''
+AIC matrix. Wake iterations (with their number defined by wakeiter kwarg) are performed according to provided freestream arguments'''
 paneller.Solid.calcpress.__doc__='''calcpress(Uinf=1.0): calculates Solid.Cps based on Solid.vbar and Solid.delphi'''
 paneller.Solid.calcforces.__doc__='''calcforces(): calculates local forces and moments on each panel, based on Solid.Cps and Solid.Cfs'''
 paneller.Solid.calc_derivative.__doc__='''calc_derivative(Uinf, a=0.0, b=0.0, p=0.0, q=0.0, r=0.0, par='a'): calculates derivatives in local pressure coefficients as array dCps, differentiated by
@@ -172,9 +225,12 @@ paneller.Solid.plotgeometry.__doc__='''plotgeometry(xlim=[], ylim=[], zlim=[], v
 if velfield==True, plots vectors correspondent to each panel\'s local velocity vector multiplier by a factor (\'factor\' kwarg) for easier visualization of velocity field'''
 paneller.Solid.plotnormals.__doc__='''plotnormals(xlim=[], ylim=[], zlim=[], factor=1.0): plots panels as wireframe and their normal vectors p.nvector, scaled by factor kwarg so as to ease visualization.
 Sets plot limits if provided in non-empty kwargs'''
-paneller.Solid.eulersolve.__doc__='''eulersolve(target=np.array([]), a=0.0, b=0.0, p=0.0, q=0.0, r=0.0, damper=0.0, Uinf=1.0, echo=True): computes complete euler solution through calls for 
-Solid.genvbar, gennvv, genaicm, solve, calcpress and calcforces. Uses given freestream parameters and outputs time duration report if echo==True'''
+paneller.Solid.eulersolve.__doc__='''eulersolve(target=np.array([]), a=0.0, b=0.0, p=0.0, q=0.0, r=0.0, damper=0.0, Uinf=1.0, echo=True, beta=0.0): computes complete euler solution through calls for 
+Solid.genvbar, gennvv, genaicm, solve, calcpress and calcforces. Uses given freestream parameters and outputs time duration report if echo==True. Applies PG correction factor beta'''
 paneller.Solid.addorder.__doc__='''addorder(self, queue, colmat, ind1, ind2): adds order for multiprocessing queue for subprocess_genaicm function to calculate AIC matrix\'s columns ind1 to ind2'''
+paneller.Solid.panel_calcSn.__doc__='''panel_calcSn(p): recalculates area and normal vector for panel p'''
+paneller.Solid.PG_apply.__doc__='''PG_apply(beta, a, b): applies PG correction to all panel and line positions (including wake panels) according to correction factor beta and freestream parameters a and b'''
+paneller.Solid.PG_remove.__doc__='''PG_remove(beta, a, b): reverses effects from Solid.PG_apply(beta, a, b) and converts AIC matrixes and self-influence matrixes to compressible values'''
 
 utils.__doc__='''
 Module containing geometry processing, list trimming and other utilities for other modules
@@ -199,7 +255,7 @@ utils.read_afl.__doc__='''read_afl(afl, afldir='', ext_append=False, header_line
 incidence=0.0, inverse=False, closed=False): reads airfoil in afl (adding extension .dat if ext_append==True), eliminating header_lines lines from Selig format file. Repanels airfoil with 
 x discretization disc, using x points as strategy(np.linspace(0.0, 1.0, disc+1)), strategy being a lambda. remove_TE_gap sets the first and last points to their mean point, eliminating trailing edge
 gap. \'True\' is indicated for inviscid calculations. if extra_intra==True, points referrent to upper and lower airfoil surface are also returned, in tuple. if inverse==True, the airfoil\'s y axis is inverted.
-Returns aflpts, array of shape (2*xdisc+1, 2)'''
+Returns aflpts, array of shape (2*xdisc+1, 2). Argument sweep compresses the airfoil's y axis dimension so as to apply sweep effect on thickness'''
 utils.wing_afl_positprocess.__doc__='''wing_afl_positprocess(afl, gamma=0.0, c=1.0, ypos=0.0, xpos=0.0, zpos=0.0): returns three-dimensionalized version of airfoil in points in array afl as returned by
 utils.read_afl. Twisted around x axis by gamma (radians), with chord c and positions in three dimensional axes by correspondent kwargs'''
 utils.trimlist.__doc__='''trimlist(n, l): trims list l to length n by reproducing its first element, if it is not empty and len(l)<n'''
@@ -213,6 +269,11 @@ utils.gen_circdefsect_coords.__doc__='''gen_circdefsect_coords(disc): returns po
 utils.gen_squaredefsect_coords.__doc__='''gen_squaredefsect_coords(disc): returns points for body.squaredefsect default body section, with goven input panel discretization'''
 utils.smooth_angle_defsect_coords.__doc__='''smooth_angle_defsect_coords(r_1x, r_2x, r_1y, r_2y, ldisc=30, thdisc=20): returns coordinates for body.smooth_angle_defsect function, with given semi-ellipsoid
 concordance radii r_1x (lower, x-axis), r_2x (upper, x-axis), r_1y and r_2y'''
+utils.Mtostream.__doc__='''Mtostream(a, b): returns a matrix for coordinate system conversion to a stream-based coordinate system'''
+utils.Mstreamtouni.__doc__='''Mstreamtouni(a, b): returns the inverse of utils.Mtostream(a, b)'''
+utils.PG_xmult.__doc__='''PG_xmult(beta, a, b): returns a matrix to multiply point arrays so as to apply PG correction over their location, based on correction factor beta=sqrt(1.0-M**2)'''
+utils.PG_inv_xmult.__doc__='''PG_inv_xmult(beta, a, b): returns the inverse of PG_xmult'''
+utils.PG_vtouni.__doc__='''PG_vtouni(beta, a, b): returns matrix to multiply velocities by so as to convert them from incompressible PG circunstances to compressible values'''
 
 control.__doc__='''
 Module for definition of controls, control axes and degrees of freedom for aicraft and wing classes.
@@ -335,10 +396,11 @@ applycontrols
 '''
 wing.wing_section.__init__.__doc__='''__init__(afldir='', c=1.0, incidence=0.0, gamma=0.0, CA_position=np.array([0.0, 0.0, 0.0]), afl='n4412', \
         header_lines=1, xdisc=10, remove_TE_gap=True, inverse=False, xstrategy=lambda x: (np.sin(pi*x-pi/2)+1)/2, closed=False, \
-            correction=None, Re=2e6):
+            correction=None, Re=2e6, sweep=0.0):
             initializes wing section with airfoil \'afl\' collected from directory afldir (if len(afldir)==0: afldir=os.getcwd()) through utils.read_afl,
             with arguments header_lines, disc=xdisc, remove_TE_gap, inverse and strategy=xstrategy. closed kwarg sets wing section to closed wingtip if True.
-            if correction!=None, a correction from module xfoil_visc is provided, and Reynolds Re should be considered to generate a viscous polar for the given section'''
+            if correction!=None, a correction from module xfoil_visc is provided, and Reynolds Re should be considered to generate a viscous polar for the given section.
+            Argument sweep compresses the airfoil's y axis dimension so as to apply sweep effect on thickness'''
 wing.wing_section.hascorrection.__doc__='''hascorrection(): returns whether or not xfoil viscous correction is available (i. e. self.correction!=None)'''
 wing.wing_section.getcorrection.__doc__='''getcorrection(Re=2e6): sets lambas self.alphas, self.Cls, self.Cds, self.Cms for wing_section instance'''
 wing.wing_section.addcontrols.__doc__='''addcontrols(controls=[], control_multipliers=[], control_axpercs=[]): add controls with percentages in chordwise direction and multipliers
@@ -567,7 +629,8 @@ __call__
 body.body_section.__init__.__doc__='''__init__(center=np.array([0.0, 0.0, 0.0]), coords=np.vstack((np.sin(np.linspace(0.0, 2*pi, 360)), \
         np.cos(np.linspace(0.0, 2*pi, 360)))).T, R=1.0, y_expand=1.0, z_expand=1.0, cubic=True): constructor of a section with coordinates based on threedimensionalization
         of yz-plane coordinates coords, with maximum dimension R, around center. Scaled in y and z axis by multipliers y_expand and z_expand. If cubic is True, the object\'s
-        polar_rule lambda function will be defined as a cubic spline. Otherwise, as a linear interpolation'''
+        polar_rule lambda function (responsible for interpolating points along the section, with polar coordinates as arguments) 
+        will be defined as a cubic spline. Otherwise, as a linear interpolation'''
 body.body_section.__call__.__doc__='''__call__(th): returns point around center, with polar coordinates in [-pi; pi], clockwise around x axis with zero on z axis, and radius as defined by
 body_section.polar_rule'''
 
@@ -605,21 +668,28 @@ patchcompose
 apply_eqflatplate
 bodypanel_plotnormals
 '''
-body.body.__init__.__doc__='''__init__(sld, sections=[], tolerance=0.00005): creates body object with sections in kwarg list, with given geometric tolerance for patch definition and bound to Solid object sld'''
-body.body.set_aircraft.__doc__='''set_aircraft(acft): binds the given non-lifting body to aircraft class instance acft'''
-body.body.find_body_intersect.__doc__='''find_body_intersect(p, u, tolerance=0.00005): find a contact point with a line defined by point p and vector u. Geometric tolerance is set for maximum distance between 
+body.body.__init__.__doc__='''__init__(sld, sections=[], tolerance=0.00005):
+creates body object with sections in kwarg list, with given geometric tolerance for patch definition and bound to Solid object sld'''
+body.body.set_aircraft.__doc__='''set_aircraft(acft):
+binds the given non-lifting body to aircraft class instance acft'''
+body.body.find_body_intersect.__doc__='''find_body_intersect(p, u, tolerance=0.00005):
+find a contact point with a line defined by point p and vector u. Geometric tolerance is set for maximum distance between 
 a panel and a point to define the latter\'s belonging to it'''
-body.body.plot_input.__doc__='''plot_input(fig=None, ax=None, show=False, xlim=[], ylim=[], zlim=[], colour='gray'): plot input body geometry as a wireframe. Use given figure and axes if not None, or create
+body.body.plot_input.__doc__='''plot_input(fig=None, ax=None, show=False, xlim=[], ylim=[], zlim=[], colour='gray'):
+plot input body geometry as a wireframe. Use given figure and axes if not None, or create
 new ones otheriwse. If show is True, plt.show() will be automatically ran. Otherwise, it must be externally executed'''
-body.body.surfinterp.__doc__='''surfinterp(th, x): returns point array, in 3D space, of a point in the non-lifting body\'s surface in x position in the x axis and th polar coordinate (in [-pi; pi], 0 at
+body.body.surfinterp.__doc__='''surfinterp(th, x):
+returns point array, in 3D space, of a point in the non-lifting body\'s surface in x position in the x axis and th polar coordinate (in [-pi; pi], 0 at
 z axis, positive orientation as clockwise around x axis)'''
-body.body.side_separate.__doc__='''side_separate(leftqueue=[], rightqueue=[], upqueue=[], lowqueue=[], xstrategy=lambda x: (np.sin(pi*x-pi/2)+1)/2, xdisc=100): returns x-axis spacing (given by
-np.interp(xstrategy(np.linspace(0.0, 1.0, xdisc+1)), np.array([0.0, 1.0]), np.array([x1, x2]))) and the number of lines occupying each x-axis interval between queued lifting surfaces.
+body.body.side_separate.__doc__='''side_separate(leftqueue=[], rightqueue=[], upqueue=[], lowqueue=[], xstrategy=lambda x: (np.sin(pi*x-pi/2)+1)/2, xdisc=100):
+returns x-axis spacing (given by np.interp(xstrategy(np.linspace(0.0, 1.0, xdisc+1)), np.array([0.0, 1.0]), np.array([x1, x2]))) 
+and the number of lines occupying each x-axis interval between queued lifting surfaces.
 returns lnlines, lxdisc, rnlines, rxdisc, unlines, uxdisc, dnlines, dxdisc. inlines designates the number of lines in intervals not occupied by lifting surfaces of queue i 
-(d for low, u for upper, r for right, l for left queue). ixdisc indicates an array giving the x-spacing control points for queue i'''
-body.body.theta_queueident.__doc__='''theta_queueident(queue, xspacing, intra=False, right=False, queueident=\'l\'): sets polar coordinates (in [-pi; pi], 0 at
-z axis, positive orientation as clockwise around x axis) for points defined by xspacing array, along a queue, following upper surface points (if intra==False, 
-lower if otherwise) of surfaces in queue list. Points in the rightmost sections of the wings contained in queue are fetched if right is True, or from the leftmost section if otherwise.
+(i being d for low, u for upper, r for right, l for left queue). ixdisc indicates an array giving the x-spacing control points for queue i'''
+body.body.theta_queueident.__doc__='''theta_queueident(queue, xspacing, intra=False, right=False, queueident=\'l\'): 
+sets polar coordinates (in [-pi; pi], 0 at z axis, positive orientation as clockwise around x axis) for points defined by xspacing array, along a queue, 
+following upper surface points (if intra==False, lower if otherwise) of surfaces in queue list. 
+Points in the rightmost sections of the wings contained in queue are fetched if right is True, or from the leftmost section if otherwise.
 queueident identifies the requested queue (d for low, u for upper, r for right, l for left queue)'''
 body.body.patchcompose.__doc__='''patchcompose(leftqueue=[], rightqueue=[], upqueue=[], leftqueue=[], xstrategy=lambda x: x, xdisc=100, thstrategy=lambda x: x, thdisc_upleft=20, thdisc_upright=20, \
     thdisc_downleft=20, thdisc_downright=20, tolerance=5e-5): compose four panel patches (upleft: between upper and left queues; downright: between lower and rightmost queues; etc.) to model the body at hand.
@@ -694,8 +764,10 @@ and viscosity corrections to moment coefficients according to the shift performe
 aircraft.aircraft.transp_to_cg.__doc__='''transp_to_CG(CG): transports the center of gravity of the aircraft at hand to point in kwarg CG, through successive calls to aircraft.transp_byvec()'''
 aircraft.aircraft.hascontrol.__doc__='''hascontrol(): evaluates whether or not the aircraft has any control surface, based on the length of its controlset dictionary. The dictionary is, in turn, generated 
 from each wing quadrant\'s control lists'''
-aircraft.aircraft.addwake.__doc__='''addwake(offset=1000.0): generates wakes for all lifting surfaces based on a given offset from the trailing edge (i. e. length of the trailing edge vortexes) and currently
-set freestream parameters. Must be ran after parameter definition and patch composing, though before Euler solution and post-processing'''
+aircraft.aircraft.addwake.__doc__='''addwake(offset=1000.0, wakedisc=1, strategy=lambda x: ((np.exp(x)-1.0)/(exp(1)-1.0))**2): 
+generates wakes for all lifting surfaces based on a given offset from the trailing edge (i. e. length of the trailing edge vortexes) and currently
+set freestream parameters. Must be ran after parameter definition and patch composing, though before Euler solution and post-processing.
+the wake is discretized in wakedisc panels queued in x-axis, with offset from trailing edge set as offset*strategy(np.linspace(0.0, 1.0, wakedisc+1))'''
 aircraft.aircraft.calcforces.__doc__='''calcforces(echo=True): calculates forces acting on the aircraft based on the Solid instance aircraft.sld\'s last ran Euler solution. If echo is True, method
 aircraft.forces_report() is ran in place. Viscous corrections are also computed in place (see variables dCX, dCY,... in the class\'s documentation)'''
 aircraft.aircraft.freestream_derivative.__doc__='''freestream_derivative(par=\'a\'): calculates derivative of unitary streamwise (u) vector and its perpendicular vector v, parallel to lift, in respect to
@@ -709,8 +781,8 @@ the controlset dictionary will be set to the value provided in pardict, in degre
 and reference dimensions'''
 aircraft.aircraft.bodies_eqflatplate_apply.__doc__='''bodies_eqflatplate_apply(rho=1.225, mu=1.72*10e-5, turbulent_criterion=Re2e5, Cf_l_rule=Blausius_Cf_l, Cf_t_rule=Prandtl_1_7th): applies equivalent flat
 plate correction for friction coefficient according to given criteria for all bodies bound to the aircraft according to current freestream parameters. Check body.apply_eqflatplate() reference for further detail'''
-aircraft.aircraft.eulersolve.__doc__='''eulersolve(echo=True, damper=0.0): runs Solid.eulersolve() with the present freestream parameters set for the aircraft. Time report is presented if echo is True.
-A Thikhonov regularization (with damping coefficient set by damper kwarg) can be performed with the AIC matrix if damper is non-zero.'''
+aircraft.aircraft.eulersolve.__doc__='''eulersolve(echo=True, damper=0.0, wakeiter=0): runs Solid.eulersolve() with the present freestream parameters set for the aircraft. Time report is presented if echo is True.
+A Thikhonov regularization (with damping coefficient set by damper kwarg) can be performed with the AIC matrix if damper is non-zero. wakeiter iterations of wake rollup are executed'''
 aircraft.aircraft.forces_report.__doc__='''forces_report(): prints all force and moment coefficients available (and their viscous corrected equivalents). If none is available, method aircraft.calcforces() is ran'''
 aircraft.aircraft.plot_input.__doc__='''plot_input(xlim=[], ylim=[], zlim=[]): plots all elements of the aircraft as wireframes for input geometry debugging. Sets aircraft.plotlim as plot limits if none
 is provided'''
