@@ -29,11 +29,32 @@ def subprocess_genaicm(queue1, queue2): #generic function to unpack AICM calc or
     AICM=toolkit.aicm_lines_gen(order[2], order[3])
     queue2.put((order[0], order[1], AICM))
 
+class WakeLine: #class encompassing info about a wake line. Made so as to compute deformation with wake rollup
+    def __init__(self, ind): #recieves signed index as input
+        self.ind=ind
+        self.v=np.zeros(3)
+        self.nabut=0
+        self.updateme=True
+        self.inverted=False
+    def addvel(self, v): #add velocity to be averaged
+        self.v+=v
+        self.nabut+=1
+
+class WakePanel:
+    def __init__(self, wl, wr, center): #add a wake panel based on adjacent lines
+        self.wl=wl
+        self.wr=wr
+        self.center=center
+        self.v=np.zeros(3)
+
 class Solid:
     def __init__(self, sldlist=[], wraparounds=[]): #solid data type
         self.panels=[]
         self.lines=[]
         self.addto=[]
+        self.wakestrips=[]
+        self.wakeline_inds=set([])
+        self.wakelines=set([])
         self.problematic=[]
         self.npanels=0
         self.nlines=0
@@ -141,56 +162,82 @@ class Solid:
         self.panels+=[Panel(llist)]
         return self.npanels-1
     def addwakepanel(self, refup, refdown, indup=0, indown=2, offsetleft=np.array([1000.0, 0.0, 0.0]), offsetright=np.array([]), \
-        tolerance=0.00005): #add line segments correspondent to panel wake, and remove "inup-th"/"indown-th" line segment in the panel's list
+        tolerance=0.00005, disc=10, strategy=lambda x: ((np.exp(x)-1.0)/(exp(1)-1.0))**2, prevleft=[], leftinvert=False, prevright=[], rightinvert=False):
+        #add line segments correspondent to panel wake, and remove "inup-th"/"indown-th" line segment in the panel's list
         if len(offsetright)==0:
             offsetright=offsetleft
         self.nwake+=1
+        wakedisc=strategy(np.linspace(0.0, 1.0, disc+1))
         if indup<len(self.panels[refup].lines):
             te_linind=self.panels[refup].lines.pop(indup)
         else:
             te_linind=self.panels[refup].wakelines.pop(indup-len(self.panels[refup].lines))
         self.panels[refup].TE_line=te_linind
-        if te_linind>0:
-            #first wake vortex
-            p0=self.lines[te_linind-1, :, 0]
-            p1=self.lines[te_linind-1, :, 0]+offsetleft
-            indsup=[self.addline(np.vstack((p0, p1)).T, tolerance=tolerance)]
-            #farfield wake vortex
-            p0=p1
-            p1=self.lines[te_linind-1, :, 1]+offsetright
-            indsup=[self.addline(np.vstack((p0, p1)).T, tolerance=tolerance)]
-            #second wake vortex
-            p0=p1
-            p1=self.lines[te_linind-1, :, 1]
-            indsup=[self.addline(np.vstack((p0, p1)).T, tolerance=tolerance)]
+        coords=self.line_getcoords(te_linind)
+        self.wakestrips+=[[]]
+        indsup=[]
+        leftlines=[]
+        rightlines=[]
+        if len(prevright)==0:
+            for i in range(disc):
+                p0=coords[:, 0]+offsetright*wakedisc[i]
+                p1=coords[:, 0]+offsetright*wakedisc[i+1]
+                indsup+=[self.addline(np.vstack((p0, p1)).T, tolerance=tolerance)+1]
+                rightlines+=[WakeLine(indsup[-1])]
         else:
-            #first wake vortex
-            p0=self.lines[-te_linind-1, :, 1]
-            p1=self.lines[-te_linind-1, :, 1]+offsetright
-            indsup=[self.addline(np.vstack((p0, p1)).T, tolerance=tolerance)]
-            #farfield wake vortex
-            p0=p1
-            p1=self.lines[-te_linind-1, :, 0]+offsetleft
-            indsup=[self.addline(np.vstack((p0, p1)).T, tolerance=tolerance)]
-            #second wake vortex
-            p0=p1
-            p1=self.lines[-te_linind-1, :, 0]
-            indsup=[self.addline(np.vstack((p0, p1)).T, tolerance=tolerance)]
+            if rightinvert:
+                rightlines=[prevright[i] for i in range(len(prevright)-1, -1, -1)]
+                indsup+=[-prevright[i].ind for i in range(len(prevright)-1, -1, -1)]
+            else:
+                indsup+=[prevright[i].ind for i in range(len(prevright))]
+                rightlines=prevright
+        if len(prevleft)==0:
+            for i in range(disc, 0, -1):
+                p0=coords[:, 1]+offsetleft*wakedisc[i]
+                p1=coords[:, 1]+offsetleft*wakedisc[i-1]
+                indsup+=[self.addline(np.vstack((p0, p1)).T, tolerance=tolerance)+1]
+                leftlines+=[WakeLine(indsup[-1])]
+        else:
+            if leftinvert:
+                leftlines=[prevleft[i] for i in range(len(prevleft)-1, -1, -1)]
+                indsup+=[-prevleft[i].ind for i in range(len(prevleft)-1, -1, -1)]
+            else:
+                indsup+=[prevleft[i].ind for i in range(len(prevleft))]
+                leftlines=prevleft
         self.panels[refup].wakelines+=indsup
+        for l in leftlines+rightlines:
+            self.wakelines.add(l)
+        for lind in indsup:
+            self.wakeline_inds.add(abs(lind)-1)
         if refdown!=-1:
             te_linind=self.panels[refdown].lines.pop(indown)
             self.panels[refdown].wakelines+=[-l for l in indsup]
             self.panels[refdown].TE_line=te_linind
-    def genwakepanels(self, wakecombs=[], wakeinds=[], offset=1000.0, a=0.0, b=0.0): #generate wake panels based in list of TE panel combinations
+        for i in range(disc):
+            center=(self.line_midpoint(rightlines[i].ind)+self.line_midpoint(leftlines[disc-i-1].ind))/2
+            self.wakestrips[-1]+=[WakePanel(leftlines[disc-i-1], rightlines[i], center)]
+        return leftlines, rightlines #return left, right wake lines
+    def genwakepanels(self, wakecombs=[], wakeinds=[], offset=1000.0, a=0.0, b=0.0, disc=10, strategy=lambda x: ((np.exp(x)-1.0)/(exp(1)-1.0))**2, \
+        prevleft=[], prevright=[], leftinvert=False, rightinvert=False):
+        #generate wake panels based in list of TE panel combinations
         #wakecombs: list of lists, first element in sublist is upper surface panel
         #wakeinds: list of lists indicating corresponding wake vortex line segment index to apply kutta condition to
         if len(wakeinds)==0:
             wakeinds=[[0, 2]]
         trimlist(len(wakecombs), wakeinds)
-        for i in range(len(wakecombs)):
+        self.addto+=[[wakecombs[0][0], wakecombs[0][1]]]
+        left, prevleft=self.addwakepanel(wakecombs[0][0], wakecombs[0][1], indup=wakeinds[0][0], indown=wakeinds[0][0], \
+            offsetleft=np.array([cos(a)*cos(b), -cos(a)*sin(b), sin(a)])*offset, disc=disc, strategy=strategy, prevleft=prevleft, leftinvert=leftinvert)
+        for i in range(1, len(wakecombs)-1):
             self.addto+=[[wakecombs[i][0], wakecombs[i][1]]]
-            self.addwakepanel(wakecombs[i][0], wakecombs[i][1], indup=wakeinds[i][0], indown=wakeinds[i][0], \
-                offsetleft=np.array([cos(a)*cos(b), -cos(a)*sin(b), sin(a)])*offset)
+            _, prevleft=self.addwakepanel(wakecombs[i][0], wakecombs[i][1], indup=wakeinds[i][0], indown=wakeinds[i][0], \
+                offsetleft=np.array([cos(a)*cos(b), -cos(a)*sin(b), sin(a)])*offset, disc=disc, strategy=strategy, \
+                    prevleft=prevleft, leftinvert=True)
+        self.addto+=[[wakecombs[-1][0], wakecombs[-1][1]]]
+        _, prevleft=self.addwakepanel(wakecombs[-1][0], wakecombs[-1][1], indup=wakeinds[-1][0], indown=wakeinds[-1][0], \
+            offsetleft=np.array([cos(a)*cos(b), -cos(a)*sin(b), sin(a)])*offset, disc=disc, strategy=strategy, \
+                prevleft=prevleft, leftinvert=True, prevright=prevright, rightinvert=rightinvert)
+        return left, prevleft
         #straight, alpha and beta defined single wake panel as default
     def panel_getcoords(self, p): #get coordinates for panel based on line vectors and lists
         if p.lines[0]<0:
@@ -209,6 +256,15 @@ class Solid:
             coords[:, 0:3]=temp
             coords[:, -1]=temp[:, -1]
         return coords
+    def panel_calcSn(self, p): #update panel area and normal vector
+        l=self.line_getcoords(p.lines[0])
+        p.nvector=np.cross(l[:, 1]-p.colpoint, l[:, 0]-p.colpoint)
+        p.S=lg.norm(p.nvector)
+        p.nvector/=p.S
+        p.S/=2
+        for i in range(1, len(p.lines)):
+            l=self.line_getcoords(p.lines[i])
+            p.S+=lg.norm(np.cross(l[:, 1]-p.colpoint, l[:, 0]-p.colpoint))/2
     def line_getcoords(self, ind): #return line coordinates in order presented in panel, based on panel.lines element
         if ind>0:
             return self.lines[ind-1, :, :]
@@ -266,7 +322,8 @@ class Solid:
                         print('WARNING: '+str(i)+' panel is not contiguous')
                         self.problematic+=[abs(l)-1 for l in self.panels[i].lines]
                         break
-    def end_preprocess(self, paninds=[], tolerance=5e-5): #calculate panel areas before they are altered by wake generation. Must be run before it, and terrible
+    def end_preprocess(self, paninds=[], tolerance=5e-5, initialize=True, check=True):
+        #calculate panel areas before they are altered by wake generation. Must be run before it, and terrible
         #consequences may arise if done otherwise
         u=np.array([0.0, 0.0, 0.0])
         v=np.array([0.0, 0.0, 0.0])
@@ -306,22 +363,22 @@ class Solid:
                 print('WARNING: panel '+str(n)+' displayed incoherent area '+str(p.S)+' with '+str(len(p.lines))+' lines')
             n+=1
         #initialize result vectors
-        self.delphi=np.array([[0.0, 0.0, 0.0]]*len(self.panels), dtype='double')
-        self.vbar=np.array([[0.0, 0.0, 0.0]]*len(self.panels), dtype='double')
-        self.nvv=np.array([0.0]*len(self.panels), dtype='double')
-        self.solution=np.array([0.0]*len(self.panels), dtype='double')
-        self.solavailable=False
-        self.Cps=np.array([0.0]*len(self.panels), dtype='double')
-        self.Cfs=np.zeros(len(self.panels), dtype='double')
-        self.forces=[]
-        self.moments=[]
+        if initialize:
+            self.delphi=np.array([[0.0, 0.0, 0.0]]*len(self.panels), dtype='double')
+            self.vbar=np.array([[0.0, 0.0, 0.0]]*len(self.panels), dtype='double')
+            self.nvv=np.array([0.0]*len(self.panels), dtype='double')
+            self.solution=np.array([0.0]*len(self.panels), dtype='double')
+            self.solavailable=False
+            self.Cps=np.array([0.0]*len(self.panels), dtype='double')
+            self.Cfs=np.zeros(len(self.panels), dtype='double')
+            self.forces=[]
+            self.moments=[]
         #adjust lines in case any is set inconsistently with respect to anti-clockwise convention in panel
-        self.lineadjust()
-        #self.iscontiguous(tolerance=tolerance)
+        if check:
+            self.lineadjust()
+            #self.iscontiguous(tolerance=tolerance)
     def gen_panline(self): #generate panel-line correspondence matrix
         self.panline_matrix=np.zeros((self.nlines, self.npanels), dtype='double')
-        conlist=[]
-        invlist=[]
         for i in range(self.npanels):
             lininds=np.array(self.panels[i].lines+self.panels[i].wakelines)
             lintemp=lininds[lininds>0]
@@ -369,6 +426,10 @@ class Solid:
         for i in range(self.npanels):
             newvec[i, :]=Uinf*np.array([cos(a)*cos(b), -cos(a)*sin(b), sin(a)], dtype='double')+np.cross(np.array([p, -q, r], dtype='double'), self.panels[i].colpoint)
         return newvec
+    def gen_wake_farfield(self, Uinf, a=0.0, b=0.0, p=0.0, q=0.0, r=0.0): #generate generic local freestream at wake panels
+        for strip in self.wakestrips:
+            for pan in strip:
+                pan.v=Uinf*np.array([cos(a)*cos(b), -cos(a)*sin(b), sin(a)], dtype='double')+np.cross(np.array([p, -q, r], dtype='double'), pan.center)
     def gen_farfield_derivative(self, Uinf, a=0.0, b=0.0, p=0.0, q=0.0, r=0.0, par='a'): #generate generic local freestream velocity dependant on parameters
         newvec=np.zeros((self.npanels, 3))
         if par=='a':
@@ -392,7 +453,7 @@ class Solid:
         return newvec
     def genvbar(self, Uinf, a=0.0, b=0.0, p=0.0, q=0.0, r=0.0): #generate freestream velocity vector. Angular velocities are raw (rad/s), not normalized by Uinf or dimensions
         self.vbar=self.gen_farfield(Uinf, a=a, b=b, p=p, q=q, r=r)
-    def gennvv(self): #Generate vector containing normal velocities
+    def gennvv(self, beta=1.0): #Generate vector containing normal velocities
         for i in range(self.npanels):
             self.nvv[i]=self.panels[i].nvector@self.vbar[i, :]
     def gen_selfinf_mat(self): #generate self-influence matrix according to Srivastava's equations, with each line's vorticity as an input
@@ -426,7 +487,8 @@ class Solid:
         self.delphi[:, 0]+=self.selfinf_mat_x@self.solution_line
         self.delphi[:, 1]+=self.selfinf_mat_y@self.solution_line
         self.delphi[:, 2]+=self.selfinf_mat_z@self.solution_line
-    def solve(self, damper=0.0, target=np.array([])): #generate Euler solution. Inverts AIC matrix with Tikhonov regularization if "damper" is set to non-zero value.
+    def solve(self, damper=0.0, target=np.array([]), wakeiter=0, Uinf=1.0, a=0.0, b=0.0, p=0.0, q=0.0, r=0.0, tolerance=1e-5, echo=True):
+        #generate Euler solution. Inverts AIC matrix with Tikhonov regularization if "damper" is set to non-zero value.
         #self.iaicm=toolkit.aicm_inversion(self.aicm, damper)
         if len(target)==0:
             target=np.zeros(self.npanels) # target: nvv aimed at, for stability derivative computation and/or viscous-inviscid coupling
@@ -439,12 +501,39 @@ class Solid:
 
         self.solution=self.iaicm@tango
         self.solution_line=self.panline_matrix@self.solution
+        self.solavailable=True
+        if wakeiter!=0:
+            wakelininds=np.array(list(self.wakeline_inds))
+            colmat=np.zeros((self.npanels, 3))
+            nvectmat=np.zeros((self.npanels, 3))
+            for i in range(len(self.panels)):
+                colmat[i, :]=self.panels[i].colpoint
+                nvectmat[i, :]=self.panels[i].nvector
+            if echo:
+                print('%18s | %5s %5s %5s' % ('Iteration', 'vcalc', 'roll', 'solve'))
+        for i in range(wakeiter): #iterate for wake generation
+            t0=tm.time()
+            self.gen_wake_farfield(Uinf=Uinf, a=a, b=b, p=p, q=q, r=r)
+            self.add_wakevels(tolerance=tolerance)
+            tvel=tm.time()
+            self.wake_rollup()
+            trol=tm.time()
+            self.aicm3_line[:, :, wakelininds]=toolkit.aicm_lines_recalc(wakelininds+1, self.lines, colmat)
+            for j in range(3):
+                self.aicm3[j, :, :]=self.aicm3_line[j, :, :]@self.panline_matrix
+            self.aicm=toolkit.aicm_norm_conv(self.aicm3, nvectmat)
+            if damper!=0.0:
+                self.iaicm=slg.inv(self.aicm.T@self.aicm+damper*np.eye(self.npanels, self.npanels))@self.aicm.T
+            else:
+                self.iaicm=slg.inv(self.aicm)
+            self.solution=self.iaicm@tango
+            self.solution_line=self.panline_matrix@self.solution
+            tsol=tm.time()
+            print('%14s %3d | %5f %5f %5f' % ('Wake teration', i, tvel-t0, trol-tvel, tsol-trol))
+            
         for i in range(3):
             self.delphi[:, i]=self.aicm3[i, :, :]@self.solution #compute local velocity due to disturbance field
-        self.gen_selfinf_mat()
         self.gen_selfinf()
-        self.solavailable=True
-        #self.calcforces()
     def calcpress(self, Uinf=1.0):
         for i in range(self.npanels):
             self.Cps[i]=(Uinf**2-(self.delphi[i, :]+self.vbar[i, :])@(self.delphi[i, :]+self.vbar[i, :]))/Uinf**2
@@ -462,6 +551,101 @@ class Solid:
         dvdksi[:, 2]+=self.aicm3[2, :, :]@dGammadksi+self.selfinf_mat_z@dGamma_linedksi
         dCps=np.array([-(2*(self.vbar[i, :]+self.delphi[i, :])@dvdksi[i, :])/Uinf**2 for i in range(self.npanels)])
         return dCps
+    def add_wakevels(self, tolerance=1e-5): #calculate velocities at wake panel control points and add to local velocity variables
+        for strip in self.wakestrips:
+            for p in strip:
+                p.v+=toolkit.get_field_influence(self.lines, self.solution_line, p.center, tolerance=tolerance)
+    def wake_rollup(self): #calculate velocity at nodes located at wake line midpoints and update the lines
+        for strip in self.wakestrips:
+            for p in strip:
+                p.wl.addvel(p.v)
+                p.wr.addvel(p.v)
+        self.wakelines_deform()
+    def wakelines_deform(self): #deform wake lines according to computed velocity at panels between them
+        for l in self.wakelines:
+            l.v/=l.nabut
+            l.inverted=self.lines[abs(l.ind)-1, 0, 0]>self.lines[abs(l.ind)-1, 0, 1]
+            l.updateme=True
+        deltaxl=0.0
+        deltaxr=0.0
+        dvl=np.zeros(3)
+        dvr=np.zeros(3)
+        origin_left=np.zeros(3)
+        origin_right=np.zeros(3)
+        newleft=np.zeros(3)
+        newright=np.zeros(3)
+        for strip in self.wakestrips:
+            origin_left=self.lines[abs(strip[0].wl.ind)-1, :, 1 if strip[0].wl.inverted else 0]
+            origin_right=self.lines[abs(strip[0].wr.ind)-1, :, 1 if strip[0].wr.inverted else 0]
+            for i in range(len(strip)):
+                if strip[i].wl.updateme:
+                    deltaxl=abs(self.lines[abs(strip[i].wl.ind)-1, 0, 1]-self.lines[abs(strip[i].wl.ind)-1, 0, 0])
+                    dvl=strip[i].wl.v*deltaxl/(strip[i].wl.v[0])
+                    newleft=origin_left+dvl
+                    if strip[i].wl.inverted:
+                        self.lines[abs(strip[i].wl.ind)-1, :, 1]=origin_left
+                        self.lines[abs(strip[i].wl.ind)-1, :, 0]=newleft
+                    else:
+                        self.lines[abs(strip[i].wl.ind)-1, :, 0]=origin_left
+                        self.lines[abs(strip[i].wl.ind)-1, :, 1]=newleft
+                    origin_left=newleft
+                    strip[i].wl.updateme=False
+                if strip[i].wr.updateme:
+                    deltaxr=abs(self.lines[abs(strip[i].wr.ind)-1, 0, 1]-self.lines[abs(strip[i].wr.ind)-1, 0, 0])
+                    dvr=strip[i].wr.v*deltaxr/(strip[i].wr.v[0])
+                    newright=origin_right+dvr
+                    if strip[i].wr.inverted:
+                        self.lines[abs(strip[i].wr.ind)-1, :, 1]=origin_right
+                        self.lines[abs(strip[i].wr.ind)-1, :, 0]=newright
+                    else:
+                        self.lines[abs(strip[i].wr.ind)-1, :, 0]=origin_right
+                        self.lines[abs(strip[i].wr.ind)-1, :, 1]=newright
+                    origin_right=newright
+                    strip[i].wr.updateme=False
+                strip[i].center=(self.line_midpoint(strip[i].wr.ind)+self.line_midpoint(strip[i].wl.ind))/2
+        for l in self.wakelines:
+            l.v=0.0
+            l.nabut=0
+    def PG_apply(self, beta, a, b): #beta indicates PG correction factor for compressibility
+        PG_apmat=PG_xmult(beta, a, b)
+        #convert lines
+        self.lines[:, 0, :]=PG_apmat[0, 0]*self.lines[:, 0, :]+PG_apmat[0, 1]*self.lines[:, 1, :]+PG_apmat[0, 2]*self.lines[:, 2, :]
+        self.lines[:, 1, :]=PG_apmat[1, 0]*self.lines[:, 0, :]+PG_apmat[1, 1]*self.lines[:, 1, :]+PG_apmat[1, 2]*self.lines[:, 2, :]
+        self.lines[:, 2, :]=PG_apmat[2, 0]*self.lines[:, 0, :]+PG_apmat[2, 1]*self.lines[:, 1, :]+PG_apmat[2, 2]*self.lines[:, 2, :]
+        #convert panels
+        for p in self.panels:
+            p.colpoint=PG_apmat@p.colpoint
+            self.panel_calcSn(p)
+        #convert wake panels
+        for strip in self.wakestrips:
+            for p in strip:
+                p.center=PG_apmat@p.center
+    def PG_remove(self, beta, a, b): #remove effect of previously imposed PG correction
+        PG_apmat=PG_inv_xmult(beta, a, b)
+        PG_apv=PG_vtouni(beta, a, b)
+        #convert lines
+        self.lines[:, 0, :]=PG_apmat[0, 0]*self.lines[:, 0, :]+PG_apmat[0, 1]*self.lines[:, 1, :]+PG_apmat[0, 2]*self.lines[:, 2, :]
+        self.lines[:, 1, :]=PG_apmat[1, 0]*self.lines[:, 0, :]+PG_apmat[1, 1]*self.lines[:, 1, :]+PG_apmat[1, 2]*self.lines[:, 2, :]
+        self.lines[:, 2, :]=PG_apmat[2, 0]*self.lines[:, 0, :]+PG_apmat[2, 1]*self.lines[:, 1, :]+PG_apmat[2, 2]*self.lines[:, 2, :]
+        #convert panels
+        for p in self.panels:
+            p.colpoint=PG_apmat@p.colpoint
+            self.panel_calcSn(p)
+        #convert wake panels
+        for strip in self.wakestrips:
+            for p in strip:
+                p.center=PG_apmat@p.center
+        #convert self influence and aicm3_line matrixes
+        self.selfinf_mat_x=PG_apv[0, 0]*self.selfinf_mat_x+PG_apv[0, 1]*self.selfinf_mat_y+PG_apv[0, 2]*self.selfinf_mat_z
+        self.selfinf_mat_y=PG_apv[1, 0]*self.selfinf_mat_x+PG_apv[1, 1]*self.selfinf_mat_y+PG_apv[1, 2]*self.selfinf_mat_z
+        self.selfinf_mat_z=PG_apv[2, 0]*self.selfinf_mat_x+PG_apv[2, 1]*self.selfinf_mat_y+PG_apv[2, 2]*self.selfinf_mat_z
+        #self.aicm remains the same
+        self.aicm3_line[0, :, :]=PG_apv[0, 0]*self.aicm3_line[0, :, :]+PG_apv[0, 1]*self.aicm3_line[1, :, :]+PG_apv[0, 2]*self.aicm3_line[2, :, :]
+        self.aicm3_line[1, :, :]=PG_apv[1, 0]*self.aicm3_line[0, :, :]+PG_apv[1, 1]*self.aicm3_line[1, :, :]+PG_apv[1, 2]*self.aicm3_line[2, :, :]
+        self.aicm3_line[2, :, :]=PG_apv[2, 0]*self.aicm3_line[0, :, :]+PG_apv[2, 1]*self.aicm3_line[1, :, :]+PG_apv[2, 2]*self.aicm3_line[2, :, :]
+        self.aicm3[0, :, :]=PG_apv[0, 0]*self.aicm3[0, :, :]+PG_apv[0, 1]*self.aicm3[1, :, :]+PG_apv[0, 2]*self.aicm3[2, :, :]
+        self.aicm3[1, :, :]=PG_apv[1, 0]*self.aicm3[0, :, :]+PG_apv[1, 1]*self.aicm3[1, :, :]+PG_apv[1, 2]*self.aicm3[2, :, :]
+        self.aicm3[2, :, :]=PG_apv[2, 0]*self.aicm3[0, :, :]+PG_apv[2, 1]*self.aicm3[1, :, :]+PG_apv[2, 2]*self.aicm3[2, :, :]
     def plotgeometry(self, xlim=[], ylim=[], zlim=[], velfield=True, factor=1.0):
         #plot geometry and local velocity vectors, either with or without wake panels
         fig=plt.figure()
@@ -513,21 +697,25 @@ class Solid:
         plt.xlabel('x')
         plt.ylabel('y')
         plt.show()
-    def eulersolve(self, target=np.array([]), a=0.0, b=0.0, p=0.0, q=0.0, r=0.0, damper=0.0, Uinf=1.0, echo=True):
+    def eulersolve(self, target=np.array([]), Uinf=1.0, beta=1.0, a=0.0, b=0.0, p=0.0, q=0.0, r=0.0, damper=0.0, echo=True, \
+        wakeiter=0, tolerance=1e-5):
         if echo:
             print('========Euler solution=======')
-            print(self.npanels, ' panels')
+            print(self.npanels, ' panels, ', self.nlines, ' lines')
         t=tm.time()
         self.genvbar(Uinf, a=a, b=b, p=p, q=q, r=r)
         self.gennvv()
+        self.PG_apply(beta, a, b)
         if echo:
             print('Pre-processing: '+str(tm.time()-t)+' s')
         t=tm.time()
         self.genaicm()
+        self.gen_selfinf_mat()
         if echo:
             print('AIC matrix generation: '+str(tm.time()-t)+' s')
         t=tm.time()
-        self.solve(damper=damper)
+        self.PG_remove(beta, a, b)
+        self.solve(damper=damper, wakeiter=wakeiter, Uinf=Uinf, a=a, b=b, p=p, q=q, r=r, tolerance=tolerance, echo=echo)
         self.calcpress(Uinf=Uinf)
         self.calcforces()
         if echo:
